from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.cohort_pool_components import CohortPoolComponents
from ...models.cohort_pool_components_error import CohortPoolComponentsError
from ...models.cohort_pool_components_not_found import CohortPoolComponentsNotFound
from ...models.cohort_pool_components_unauthorized import (
    CohortPoolComponentsUnauthorized,
)
from ...types import UNSET, Response


def _get_kwargs(
    *,
    client: Client,
    cohort: str,
) -> Dict[str, Any]:
    url = "{}/api/v1/cohorts/pools".format(client.base_url)

    headers: Dict[str, Any] = client.get_headers()

    params: Dict[str, Any] = {
        "cohort": cohort,
    }
    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "url": url,
        "headers": headers,
        "cookies": client.get_cookies(),
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[
    Union[
        CohortPoolComponents,
        CohortPoolComponentsUnauthorized,
        CohortPoolComponentsNotFound,
        CohortPoolComponentsError,
    ]
]:
    if response.status_code == 200:
        response_200 = CohortPoolComponents.from_dict(response.json())

        return response_200
    if response.status_code == 401:
        response_401 = CohortPoolComponentsUnauthorized.from_dict(response.json())

        return response_401
    if response.status_code == 404:
        response_404 = CohortPoolComponentsNotFound.from_dict(response.json())

        return response_404
    if response.status_code == 500:
        response_500 = CohortPoolComponentsError.from_dict(response.json())

        return response_500
    return None


def _build_response(
    *, response: httpx.Response
) -> Response[
    Union[
        CohortPoolComponents,
        CohortPoolComponentsUnauthorized,
        CohortPoolComponentsNotFound,
        CohortPoolComponentsError,
    ]
]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    cohort: str,
) -> Response[
    Union[
        CohortPoolComponents,
        CohortPoolComponentsUnauthorized,
        CohortPoolComponentsNotFound,
        CohortPoolComponentsError,
    ]
]:
    kwargs = _get_kwargs(
        client=client,
        cohort=cohort,
    )

    response = httpx.get(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    cohort: str,
) -> Optional[
    Union[
        CohortPoolComponents,
        CohortPoolComponentsUnauthorized,
        CohortPoolComponentsNotFound,
        CohortPoolComponentsError,
    ]
]:
    """


    Args:
        cohort: Cohort we want to fetch pool components for.

    Returns:
        Either a valid response or an unauthorized, not found or other error.
    """

    return sync_detailed(
        client=client,
        cohort=cohort,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    cohort: str,
) -> Response[
    Union[
        CohortPoolComponents,
        CohortPoolComponentsUnauthorized,
        CohortPoolComponentsNotFound,
        CohortPoolComponentsError,
    ]
]:
    kwargs = _get_kwargs(
        client=client,
        cohort=cohort,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    cohort: str,
) -> Optional[
    Union[
        CohortPoolComponents,
        CohortPoolComponentsUnauthorized,
        CohortPoolComponentsNotFound,
        CohortPoolComponentsError,
    ]
]:
    """  """

    return (
        await asyncio_detailed(
            client=client,
            cohort=cohort,
        )
    ).parsed
