from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.cohort_search import CohortSearch
from ...models.cohort_search_error import CohortSearchError
from ...models.cohort_search_not_found import CohortSearchNotFound
from ...models.cohort_search_unauthorized import CohortSearchUnauthorized
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: Client,
    ticker: Union[Unset, str] = UNSET,
    coupon: Union[Unset, float] = UNSET,
    vintage: Union[Unset, int] = UNSET,
    type: Union[Unset, str] = UNSET,
) -> Dict[str, Any]:
    url = "{}/api/v1/cohorts/search".format(client.base_url)

    headers: Dict[str, Any] = client.get_headers()

    params: Dict[str, Any] = {
        "ticker": ticker,
        "coupon": coupon,
        "vintage": vintage,
        "type": type,
    }
    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "url": url,
        "headers": headers,
        "cookies": client.get_cookies(),
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[
    Union[
        CohortSearch, CohortSearchUnauthorized, CohortSearchNotFound, CohortSearchError
    ]
]:
    if response.status_code == 200:
        response_200 = CohortSearch.from_dict(response.json())

        return response_200
    if response.status_code == 401:
        response_401 = CohortSearchUnauthorized.from_dict(response.json())

        return response_401
    if response.status_code == 404:
        response_404 = CohortSearchNotFound.from_dict(response.json())

        return response_404
    if response.status_code == 500:
        response_500 = CohortSearchError.from_dict(response.json())

        return response_500
    return None


def _build_response(
    *, response: httpx.Response
) -> Response[
    Union[
        CohortSearch, CohortSearchUnauthorized, CohortSearchNotFound, CohortSearchError
    ]
]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    ticker: Union[Unset, str] = UNSET,
    coupon: Union[Unset, float] = UNSET,
    vintage: Union[Unset, int] = UNSET,
    type: Union[Unset, str] = UNSET,
) -> Response[
    Union[
        CohortSearch, CohortSearchUnauthorized, CohortSearchNotFound, CohortSearchError
    ]
]:
    kwargs = _get_kwargs(
        client=client,
        ticker=ticker,
        coupon=coupon,
        vintage=vintage,
        type=type,
    )

    response = httpx.get(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    ticker: Union[Unset, str] = UNSET,
    coupon: Union[Unset, float] = UNSET,
    vintage: Union[Unset, int] = UNSET,
    type: Union[Unset, str] = UNSET,
) -> Optional[
    Union[
        CohortSearch, CohortSearchUnauthorized, CohortSearchNotFound, CohortSearchError
    ]
]:
    """


    Args:
        ticker: None
        coupon: None
        vintage: None
        type: None

    Returns:
        Either a valid response or an unauthorized, not found or other error.
    """

    return sync_detailed(
        client=client,
        ticker=ticker,
        coupon=coupon,
        vintage=vintage,
        type=type,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    ticker: Union[Unset, str] = UNSET,
    coupon: Union[Unset, float] = UNSET,
    vintage: Union[Unset, int] = UNSET,
    type: Union[Unset, str] = UNSET,
) -> Response[
    Union[
        CohortSearch, CohortSearchUnauthorized, CohortSearchNotFound, CohortSearchError
    ]
]:
    kwargs = _get_kwargs(
        client=client,
        ticker=ticker,
        coupon=coupon,
        vintage=vintage,
        type=type,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    ticker: Union[Unset, str] = UNSET,
    coupon: Union[Unset, float] = UNSET,
    vintage: Union[Unset, int] = UNSET,
    type: Union[Unset, str] = UNSET,
) -> Optional[
    Union[
        CohortSearch, CohortSearchUnauthorized, CohortSearchNotFound, CohortSearchError
    ]
]:
    """  """

    return (
        await asyncio_detailed(
            client=client,
            ticker=ticker,
            coupon=coupon,
            vintage=vintage,
            type=type,
        )
    ).parsed
