from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.cohort_prediction import CohortPrediction
from ...models.cohort_prediction_bad_request import CohortPredictionBadRequest
from ...models.cohort_prediction_error import CohortPredictionError
from ...models.cohort_prediction_not_found import CohortPredictionNotFound
from ...models.cohort_prediction_unauthorized import CohortPredictionUnauthorized
from ...models.prediction_service_get_cohort_convention import (
    PredictionService_GetCohortConvention,
)
from ...models.prediction_service_get_cohort_scenario import (
    PredictionService_GetCohortScenario,
)
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: Client,
    cohort: str,
    as_ofyear: Union[Unset, int] = UNSET,
    as_ofmonth: Union[Unset, int] = UNSET,
    as_ofday: Union[Unset, int] = UNSET,
    convention: Union[
        Unset, PredictionService_GetCohortConvention
    ] = PredictionService_GetCohortConvention.PERIOD,
    scenario: Union[
        Unset, PredictionService_GetCohortScenario
    ] = PredictionService_GetCohortScenario.BASELINE,
) -> Dict[str, Any]:
    url = "{}/api/v1/predictions/cohort/latest".format(client.base_url)

    headers: Dict[str, Any] = client.get_headers()

    json_convention: Union[Unset, PredictionService_GetCohortConvention] = UNSET
    if not isinstance(convention, Unset):
        json_convention = convention

    json_scenario: Union[Unset, PredictionService_GetCohortScenario] = UNSET
    if not isinstance(scenario, Unset):
        json_scenario = scenario

    params: Dict[str, Any] = {
        "cohort": cohort,
        "asOf.year": as_ofyear,
        "asOf.month": as_ofmonth,
        "asOf.day": as_ofday,
        "convention": json_convention,
        "scenario": json_scenario,
    }
    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "url": url,
        "headers": headers,
        "cookies": client.get_cookies(),
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[
    Union[
        CohortPrediction,
        CohortPredictionBadRequest,
        CohortPredictionUnauthorized,
        CohortPredictionNotFound,
        CohortPredictionError,
    ]
]:
    if response.status_code == 200:
        response_200 = CohortPrediction.from_dict(response.json())

        return response_200
    if response.status_code == 400:
        response_400 = CohortPredictionBadRequest.from_dict(response.json())

        return response_400
    if response.status_code == 401:
        response_401 = CohortPredictionUnauthorized.from_dict(response.json())

        return response_401
    if response.status_code == 404:
        response_404 = CohortPredictionNotFound.from_dict(response.json())

        return response_404
    if response.status_code == 500:
        response_500 = CohortPredictionError.from_dict(response.json())

        return response_500
    return None


def _build_response(
    *, response: httpx.Response
) -> Response[
    Union[
        CohortPrediction,
        CohortPredictionBadRequest,
        CohortPredictionUnauthorized,
        CohortPredictionNotFound,
        CohortPredictionError,
    ]
]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    cohort: str,
    as_ofyear: Union[Unset, int] = UNSET,
    as_ofmonth: Union[Unset, int] = UNSET,
    as_ofday: Union[Unset, int] = UNSET,
    convention: Union[
        Unset, PredictionService_GetCohortConvention
    ] = PredictionService_GetCohortConvention.PERIOD,
    scenario: Union[
        Unset, PredictionService_GetCohortScenario
    ] = PredictionService_GetCohortScenario.BASELINE,
) -> Response[
    Union[
        CohortPrediction,
        CohortPredictionBadRequest,
        CohortPredictionUnauthorized,
        CohortPredictionNotFound,
        CohortPredictionError,
    ]
]:
    kwargs = _get_kwargs(
        client=client,
        cohort=cohort,
        as_ofyear=as_ofyear,
        as_ofmonth=as_ofmonth,
        as_ofday=as_ofday,
        convention=convention,
        scenario=scenario,
    )

    response = httpx.get(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    cohort: str,
    as_ofyear: Union[Unset, int] = UNSET,
    as_ofmonth: Union[Unset, int] = UNSET,
    as_ofday: Union[Unset, int] = UNSET,
    convention: Union[
        Unset, PredictionService_GetCohortConvention
    ] = PredictionService_GetCohortConvention.PERIOD,
    scenario: Union[
        Unset, PredictionService_GetCohortScenario
    ] = PredictionService_GetCohortScenario.BASELINE,
) -> Optional[
    Union[
        CohortPrediction,
        CohortPredictionBadRequest,
        CohortPredictionUnauthorized,
        CohortPredictionNotFound,
        CohortPredictionError,
    ]
]:
    """
    Fetch prepayment prediction for a single cohort for an as-of date, default to today.

    Args:
        cohort: Cohort name to fetch aggregated prepayment prediction.
        as_ofyear: Year of the date. Must be from 1 to 9999, or 0 to specify a date without a year
        as_ofmonth: Month of a year. Must be from 1 to 12, or 0 to specify a year without a month and day.
        as_ofday: Day of a month. Must be from 1 to 31 and valid for the year and month, or 0 to specify a year by itself or a year and month where the day isn't significant.Month of a year. Must be from 1 to 12, or 0 to specify a year without a month and day.
        convention: Convention to use for the date, can be PERIOD or FACTOR_DATE.
        scenario: None

    Returns:
        Either a valid response or an unauthorized, not found or other error.
    """

    return sync_detailed(
        client=client,
        cohort=cohort,
        as_ofyear=as_ofyear,
        as_ofmonth=as_ofmonth,
        as_ofday=as_ofday,
        convention=convention,
        scenario=scenario,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    cohort: str,
    as_ofyear: Union[Unset, int] = UNSET,
    as_ofmonth: Union[Unset, int] = UNSET,
    as_ofday: Union[Unset, int] = UNSET,
    convention: Union[
        Unset, PredictionService_GetCohortConvention
    ] = PredictionService_GetCohortConvention.PERIOD,
    scenario: Union[
        Unset, PredictionService_GetCohortScenario
    ] = PredictionService_GetCohortScenario.BASELINE,
) -> Response[
    Union[
        CohortPrediction,
        CohortPredictionBadRequest,
        CohortPredictionUnauthorized,
        CohortPredictionNotFound,
        CohortPredictionError,
    ]
]:
    kwargs = _get_kwargs(
        client=client,
        cohort=cohort,
        as_ofyear=as_ofyear,
        as_ofmonth=as_ofmonth,
        as_ofday=as_ofday,
        convention=convention,
        scenario=scenario,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    cohort: str,
    as_ofyear: Union[Unset, int] = UNSET,
    as_ofmonth: Union[Unset, int] = UNSET,
    as_ofday: Union[Unset, int] = UNSET,
    convention: Union[
        Unset, PredictionService_GetCohortConvention
    ] = PredictionService_GetCohortConvention.PERIOD,
    scenario: Union[
        Unset, PredictionService_GetCohortScenario
    ] = PredictionService_GetCohortScenario.BASELINE,
) -> Optional[
    Union[
        CohortPrediction,
        CohortPredictionBadRequest,
        CohortPredictionUnauthorized,
        CohortPredictionNotFound,
        CohortPredictionError,
    ]
]:
    """ Fetch prepayment prediction for a single cohort for an as-of date, default to today. """

    return (
        await asyncio_detailed(
            client=client,
            cohort=cohort,
            as_ofyear=as_ofyear,
            as_ofmonth=as_ofmonth,
            as_ofday=as_ofday,
            convention=convention,
            scenario=scenario,
        )
    ).parsed
