from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.cohort_look_ahead_prediction import CohortLookAheadPrediction
from ...models.cohort_look_ahead_prediction_bad_request import (
    CohortLookAheadPredictionBadRequest,
)
from ...models.cohort_look_ahead_prediction_error import CohortLookAheadPredictionError
from ...models.cohort_look_ahead_prediction_not_found import (
    CohortLookAheadPredictionNotFound,
)
from ...models.cohort_look_ahead_prediction_unauthorized import (
    CohortLookAheadPredictionUnauthorized,
)
from ...models.prediction_service_get_cohort_look_ahead_ahead import (
    PredictionService_GetCohortLookAheadAhead,
)
from ...models.prediction_service_get_cohort_look_ahead_convention import (
    PredictionService_GetCohortLookAheadConvention,
)
from ...models.prediction_service_get_cohort_look_ahead_scenario import (
    PredictionService_GetCohortLookAheadScenario,
)
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: Client,
    cohort: str,
    fromyear: Union[Unset, int] = UNSET,
    frommonth: Union[Unset, int] = UNSET,
    fromday: Union[Unset, int] = UNSET,
    convention: Union[
        Unset, PredictionService_GetCohortLookAheadConvention
    ] = PredictionService_GetCohortLookAheadConvention.PERIOD,
    scenario: Union[
        Unset, PredictionService_GetCohortLookAheadScenario
    ] = PredictionService_GetCohortLookAheadScenario.BASELINE,
    ahead: Union[
        Unset, PredictionService_GetCohortLookAheadAhead
    ] = PredictionService_GetCohortLookAheadAhead.NONE,
) -> Dict[str, Any]:
    url = "{}/api/v1/predictions/cohort/look_ahead".format(client.base_url)

    headers: Dict[str, Any] = client.get_headers()

    json_convention: Union[
        Unset, PredictionService_GetCohortLookAheadConvention
    ] = UNSET
    if not isinstance(convention, Unset):
        json_convention = convention

    json_scenario: Union[Unset, PredictionService_GetCohortLookAheadScenario] = UNSET
    if not isinstance(scenario, Unset):
        json_scenario = scenario

    json_ahead: Union[Unset, PredictionService_GetCohortLookAheadAhead] = UNSET
    if not isinstance(ahead, Unset):
        json_ahead = ahead

    params: Dict[str, Any] = {
        "cohort": cohort,
        "from.year": fromyear,
        "from.month": frommonth,
        "from.day": fromday,
        "convention": json_convention,
        "scenario": json_scenario,
        "ahead": json_ahead,
    }
    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "url": url,
        "headers": headers,
        "cookies": client.get_cookies(),
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[
    Union[
        CohortLookAheadPrediction,
        CohortLookAheadPredictionBadRequest,
        CohortLookAheadPredictionUnauthorized,
        CohortLookAheadPredictionNotFound,
        CohortLookAheadPredictionError,
    ]
]:
    if response.status_code == 200:
        response_200 = CohortLookAheadPrediction.from_dict(response.json())

        return response_200
    if response.status_code == 400:
        response_400 = CohortLookAheadPredictionBadRequest.from_dict(response.json())

        return response_400
    if response.status_code == 401:
        response_401 = CohortLookAheadPredictionUnauthorized.from_dict(response.json())

        return response_401
    if response.status_code == 404:
        response_404 = CohortLookAheadPredictionNotFound.from_dict(response.json())

        return response_404
    if response.status_code == 500:
        response_500 = CohortLookAheadPredictionError.from_dict(response.json())

        return response_500
    return None


def _build_response(
    *, response: httpx.Response
) -> Response[
    Union[
        CohortLookAheadPrediction,
        CohortLookAheadPredictionBadRequest,
        CohortLookAheadPredictionUnauthorized,
        CohortLookAheadPredictionNotFound,
        CohortLookAheadPredictionError,
    ]
]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    cohort: str,
    fromyear: Union[Unset, int] = UNSET,
    frommonth: Union[Unset, int] = UNSET,
    fromday: Union[Unset, int] = UNSET,
    convention: Union[
        Unset, PredictionService_GetCohortLookAheadConvention
    ] = PredictionService_GetCohortLookAheadConvention.PERIOD,
    scenario: Union[
        Unset, PredictionService_GetCohortLookAheadScenario
    ] = PredictionService_GetCohortLookAheadScenario.BASELINE,
    ahead: Union[
        Unset, PredictionService_GetCohortLookAheadAhead
    ] = PredictionService_GetCohortLookAheadAhead.NONE,
) -> Response[
    Union[
        CohortLookAheadPrediction,
        CohortLookAheadPredictionBadRequest,
        CohortLookAheadPredictionUnauthorized,
        CohortLookAheadPredictionNotFound,
        CohortLookAheadPredictionError,
    ]
]:
    kwargs = _get_kwargs(
        client=client,
        cohort=cohort,
        fromyear=fromyear,
        frommonth=frommonth,
        fromday=fromday,
        convention=convention,
        scenario=scenario,
        ahead=ahead,
    )

    response = httpx.get(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    cohort: str,
    fromyear: Union[Unset, int] = UNSET,
    frommonth: Union[Unset, int] = UNSET,
    fromday: Union[Unset, int] = UNSET,
    convention: Union[
        Unset, PredictionService_GetCohortLookAheadConvention
    ] = PredictionService_GetCohortLookAheadConvention.PERIOD,
    scenario: Union[
        Unset, PredictionService_GetCohortLookAheadScenario
    ] = PredictionService_GetCohortLookAheadScenario.BASELINE,
    ahead: Union[
        Unset, PredictionService_GetCohortLookAheadAhead
    ] = PredictionService_GetCohortLookAheadAhead.NONE,
) -> Optional[
    Union[
        CohortLookAheadPrediction,
        CohortLookAheadPredictionBadRequest,
        CohortLookAheadPredictionUnauthorized,
        CohortLookAheadPredictionNotFound,
        CohortLookAheadPredictionError,
    ]
]:
    """
    Fetch historical cohort prepayment predictions for a given look-ahead.

    Args:
        cohort: Cohort name to fetch aggregated historical look ahead prepayment prediction.
        fromyear: Year of the date. Must be from 1 to 9999, or 0 to specify a date without a year
        frommonth: Month of a year. Must be from 1 to 12, or 0 to specify a year without a month and day.
        fromday: Day of a month. Must be from 1 to 31 and valid for the year and month, or 0 to specify a year by itself or a year and month where the day isn't significant.Month of a year. Must be from 1 to 12, or 0 to specify a year without a month and day.
        convention: Convention to use for the date, can be PERIOD or FACTOR_DATE.
        scenario: None
        ahead: None

    Returns:
        Either a valid response or an unauthorized, not found or other error.
    """

    return sync_detailed(
        client=client,
        cohort=cohort,
        fromyear=fromyear,
        frommonth=frommonth,
        fromday=fromday,
        convention=convention,
        scenario=scenario,
        ahead=ahead,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    cohort: str,
    fromyear: Union[Unset, int] = UNSET,
    frommonth: Union[Unset, int] = UNSET,
    fromday: Union[Unset, int] = UNSET,
    convention: Union[
        Unset, PredictionService_GetCohortLookAheadConvention
    ] = PredictionService_GetCohortLookAheadConvention.PERIOD,
    scenario: Union[
        Unset, PredictionService_GetCohortLookAheadScenario
    ] = PredictionService_GetCohortLookAheadScenario.BASELINE,
    ahead: Union[
        Unset, PredictionService_GetCohortLookAheadAhead
    ] = PredictionService_GetCohortLookAheadAhead.NONE,
) -> Response[
    Union[
        CohortLookAheadPrediction,
        CohortLookAheadPredictionBadRequest,
        CohortLookAheadPredictionUnauthorized,
        CohortLookAheadPredictionNotFound,
        CohortLookAheadPredictionError,
    ]
]:
    kwargs = _get_kwargs(
        client=client,
        cohort=cohort,
        fromyear=fromyear,
        frommonth=frommonth,
        fromday=fromday,
        convention=convention,
        scenario=scenario,
        ahead=ahead,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    cohort: str,
    fromyear: Union[Unset, int] = UNSET,
    frommonth: Union[Unset, int] = UNSET,
    fromday: Union[Unset, int] = UNSET,
    convention: Union[
        Unset, PredictionService_GetCohortLookAheadConvention
    ] = PredictionService_GetCohortLookAheadConvention.PERIOD,
    scenario: Union[
        Unset, PredictionService_GetCohortLookAheadScenario
    ] = PredictionService_GetCohortLookAheadScenario.BASELINE,
    ahead: Union[
        Unset, PredictionService_GetCohortLookAheadAhead
    ] = PredictionService_GetCohortLookAheadAhead.NONE,
) -> Optional[
    Union[
        CohortLookAheadPrediction,
        CohortLookAheadPredictionBadRequest,
        CohortLookAheadPredictionUnauthorized,
        CohortLookAheadPredictionNotFound,
        CohortLookAheadPredictionError,
    ]
]:
    """ Fetch historical cohort prepayment predictions for a given look-ahead. """

    return (
        await asyncio_detailed(
            client=client,
            cohort=cohort,
            fromyear=fromyear,
            frommonth=frommonth,
            fromday=fromday,
            convention=convention,
            scenario=scenario,
            ahead=ahead,
        )
    ).parsed
