from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.pool_prediction_request import PoolPredictionRequest
from ...models.pool_predictions import PoolPredictions
from ...models.pool_predictions_bad_request import PoolPredictionsBadRequest
from ...models.pool_predictions_error import PoolPredictionsError
from ...models.pool_predictions_not_found import PoolPredictionsNotFound
from ...models.pool_predictions_unauthorized import PoolPredictionsUnauthorized
from ...types import Response


def _get_kwargs(
    *,
    client: Client,
    json_body: PoolPredictionRequest,
) -> Dict[str, Any]:
    url = "{}/api/v1/predictions/pools/latest".format(client.base_url)

    headers: Dict[str, Any] = client.get_headers()

    json_json_body = json_body.to_dict()

    return {
        "url": url,
        "headers": headers,
        "cookies": client.get_cookies(),
        "timeout": client.get_timeout(),
        "json": json_json_body,
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[
    Union[
        PoolPredictions,
        PoolPredictionsBadRequest,
        PoolPredictionsUnauthorized,
        PoolPredictionsNotFound,
        PoolPredictionsError,
    ]
]:
    if response.status_code == 200:
        response_200 = PoolPredictions.from_dict(response.json())

        return response_200
    if response.status_code == 400:
        response_400 = PoolPredictionsBadRequest.from_dict(response.json())

        return response_400
    if response.status_code == 401:
        response_401 = PoolPredictionsUnauthorized.from_dict(response.json())

        return response_401
    if response.status_code == 404:
        response_404 = PoolPredictionsNotFound.from_dict(response.json())

        return response_404
    if response.status_code == 500:
        response_500 = PoolPredictionsError.from_dict(response.json())

        return response_500
    return None


def _build_response(
    *, response: httpx.Response
) -> Response[
    Union[
        PoolPredictions,
        PoolPredictionsBadRequest,
        PoolPredictionsUnauthorized,
        PoolPredictionsNotFound,
        PoolPredictionsError,
    ]
]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    json_body: PoolPredictionRequest,
) -> Response[
    Union[
        PoolPredictions,
        PoolPredictionsBadRequest,
        PoolPredictionsUnauthorized,
        PoolPredictionsNotFound,
        PoolPredictionsError,
    ]
]:
    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
    )

    response = httpx.post(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    json_body: PoolPredictionRequest,
) -> Optional[
    Union[
        PoolPredictions,
        PoolPredictionsBadRequest,
        PoolPredictionsUnauthorized,
        PoolPredictionsNotFound,
        PoolPredictionsError,
    ]
]:
    """
    Fetch individual pool prepayment predictions for an as-of date, default to today.

    Args:

    Returns:
        Either a valid response or an unauthorized, not found or other error.
    """

    return sync_detailed(
        client=client,
        json_body=json_body,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    json_body: PoolPredictionRequest,
) -> Response[
    Union[
        PoolPredictions,
        PoolPredictionsBadRequest,
        PoolPredictionsUnauthorized,
        PoolPredictionsNotFound,
        PoolPredictionsError,
    ]
]:
    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.post(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    json_body: PoolPredictionRequest,
) -> Optional[
    Union[
        PoolPredictions,
        PoolPredictionsBadRequest,
        PoolPredictionsUnauthorized,
        PoolPredictionsNotFound,
        PoolPredictionsError,
    ]
]:
    """ Fetch individual pool prepayment predictions for an as-of date, default to today. """

    return (
        await asyncio_detailed(
            client=client,
            json_body=json_body,
        )
    ).parsed
