from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.pool_look_ahead_prediction_request import PoolLookAheadPredictionRequest
from ...models.pool_look_ahead_predictions import PoolLookAheadPredictions
from ...models.pool_look_ahead_predictions_bad_request import (
    PoolLookAheadPredictionsBadRequest,
)
from ...models.pool_look_ahead_predictions_error import PoolLookAheadPredictionsError
from ...models.pool_look_ahead_predictions_not_found import (
    PoolLookAheadPredictionsNotFound,
)
from ...models.pool_look_ahead_predictions_unauthorized import (
    PoolLookAheadPredictionsUnauthorized,
)
from ...types import Response


def _get_kwargs(
    *,
    client: Client,
    json_body: PoolLookAheadPredictionRequest,
) -> Dict[str, Any]:
    url = "{}/api/v1/predictions/pools/look_ahead".format(client.base_url)

    headers: Dict[str, Any] = client.get_headers()

    json_json_body = json_body.to_dict()

    return {
        "url": url,
        "headers": headers,
        "cookies": client.get_cookies(),
        "timeout": client.get_timeout(),
        "json": json_json_body,
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[
    Union[
        PoolLookAheadPredictions,
        PoolLookAheadPredictionsBadRequest,
        PoolLookAheadPredictionsUnauthorized,
        PoolLookAheadPredictionsNotFound,
        PoolLookAheadPredictionsError,
    ]
]:
    if response.status_code == 200:
        response_200 = PoolLookAheadPredictions.from_dict(response.json())

        return response_200
    if response.status_code == 400:
        response_400 = PoolLookAheadPredictionsBadRequest.from_dict(response.json())

        return response_400
    if response.status_code == 401:
        response_401 = PoolLookAheadPredictionsUnauthorized.from_dict(response.json())

        return response_401
    if response.status_code == 404:
        response_404 = PoolLookAheadPredictionsNotFound.from_dict(response.json())

        return response_404
    if response.status_code == 500:
        response_500 = PoolLookAheadPredictionsError.from_dict(response.json())

        return response_500
    return None


def _build_response(
    *, response: httpx.Response
) -> Response[
    Union[
        PoolLookAheadPredictions,
        PoolLookAheadPredictionsBadRequest,
        PoolLookAheadPredictionsUnauthorized,
        PoolLookAheadPredictionsNotFound,
        PoolLookAheadPredictionsError,
    ]
]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    json_body: PoolLookAheadPredictionRequest,
) -> Response[
    Union[
        PoolLookAheadPredictions,
        PoolLookAheadPredictionsBadRequest,
        PoolLookAheadPredictionsUnauthorized,
        PoolLookAheadPredictionsNotFound,
        PoolLookAheadPredictionsError,
    ]
]:
    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
    )

    response = httpx.post(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    json_body: PoolLookAheadPredictionRequest,
) -> Optional[
    Union[
        PoolLookAheadPredictions,
        PoolLookAheadPredictionsBadRequest,
        PoolLookAheadPredictionsUnauthorized,
        PoolLookAheadPredictionsNotFound,
        PoolLookAheadPredictionsError,
    ]
]:
    """
    Fetch individual historical pool prepayment predictions for a given look-ahead.

    Args:

    Returns:
        Either a valid response or an unauthorized, not found or other error.
    """

    return sync_detailed(
        client=client,
        json_body=json_body,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    json_body: PoolLookAheadPredictionRequest,
) -> Response[
    Union[
        PoolLookAheadPredictions,
        PoolLookAheadPredictionsBadRequest,
        PoolLookAheadPredictionsUnauthorized,
        PoolLookAheadPredictionsNotFound,
        PoolLookAheadPredictionsError,
    ]
]:
    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.post(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    json_body: PoolLookAheadPredictionRequest,
) -> Optional[
    Union[
        PoolLookAheadPredictions,
        PoolLookAheadPredictionsBadRequest,
        PoolLookAheadPredictionsUnauthorized,
        PoolLookAheadPredictionsNotFound,
        PoolLookAheadPredictionsError,
    ]
]:
    """ Fetch individual historical pool prepayment predictions for a given look-ahead. """

    return (
        await asyncio_detailed(
            client=client,
            json_body=json_body,
        )
    ).parsed
