from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.search import Search
from ...models.search_bad_request import SearchBadRequest
from ...models.search_error import SearchError
from ...models.search_not_found import SearchNotFound
from ...models.search_service_search_json_body import SearchService_SearchJsonBody
from ...models.search_unauthorized import SearchUnauthorized
from ...types import Response


def _get_kwargs(
    *,
    client: Client,
    json_body: SearchService_SearchJsonBody,
) -> Dict[str, Any]:
    url = "{}/api/v1/search/search".format(client.base_url)

    headers: Dict[str, Any] = client.get_headers()

    json_json_body = json_body.to_dict()

    return {
        "url": url,
        "headers": headers,
        "cookies": client.get_cookies(),
        "timeout": client.get_timeout(),
        "json": json_json_body,
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[
    Union[Search, SearchBadRequest, SearchUnauthorized, SearchNotFound, SearchError]
]:
    if response.status_code == 200:
        response_200 = Search.from_dict(response.json())

        return response_200
    if response.status_code == 400:
        response_400 = SearchBadRequest.from_dict(response.json())

        return response_400
    if response.status_code == 401:
        response_401 = SearchUnauthorized.from_dict(response.json())

        return response_401
    if response.status_code == 404:
        response_404 = SearchNotFound.from_dict(response.json())

        return response_404
    if response.status_code == 500:
        response_500 = SearchError.from_dict(response.json())

        return response_500
    return None


def _build_response(
    *, response: httpx.Response
) -> Response[
    Union[Search, SearchBadRequest, SearchUnauthorized, SearchNotFound, SearchError]
]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    json_body: SearchService_SearchJsonBody,
) -> Response[
    Union[Search, SearchBadRequest, SearchUnauthorized, SearchNotFound, SearchError]
]:
    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
    )

    response = httpx.post(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    json_body: SearchService_SearchJsonBody,
) -> Optional[
    Union[Search, SearchBadRequest, SearchUnauthorized, SearchNotFound, SearchError]
]:
    """
    Search for pools within a cohort universe.

    Args:

    Returns:
        Either a valid response or an unauthorized, not found or other error.
    """

    return sync_detailed(
        client=client,
        json_body=json_body,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    json_body: SearchService_SearchJsonBody,
) -> Response[
    Union[Search, SearchBadRequest, SearchUnauthorized, SearchNotFound, SearchError]
]:
    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.post(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    json_body: SearchService_SearchJsonBody,
) -> Optional[
    Union[Search, SearchBadRequest, SearchUnauthorized, SearchNotFound, SearchError]
]:
    """ Search for pools within a cohort universe. """

    return (
        await asyncio_detailed(
            client=client,
            json_body=json_body,
        )
    ).parsed
