from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.characteristic_date import CharacteristicDate
from ..types import UNSET, Unset

T = TypeVar("T", bound="CharacteristicItem")


@attr.s(auto_attribs=True)
class CharacteristicItem:
    """ Pool Current Characteristics: security factor, WAC, WALA, WAM, SMM. """

    cusip: Union[Unset, str] = UNSET
    date: Union[CharacteristicDate, Unset] = UNSET
    factor: Union[Unset, float] = UNSET
    smm: Union[Unset, float] = UNSET
    wac: Union[Unset, float] = UNSET
    wala: Union[Unset, int] = UNSET
    wam: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        cusip = self.cusip
        date: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.date, Unset):
            date = self.date.to_dict()

        factor = self.factor
        smm = self.smm
        wac = self.wac
        wala = self.wala
        wam = self.wam

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if cusip is not UNSET:
            field_dict["cusip"] = cusip
        if date is not UNSET:
            field_dict["date"] = date
        if factor is not UNSET:
            field_dict["factor"] = factor
        if smm is not UNSET:
            field_dict["smm"] = smm
        if wac is not UNSET:
            field_dict["wac"] = wac
        if wala is not UNSET:
            field_dict["wala"] = wala
        if wam is not UNSET:
            field_dict["wam"] = wam

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        cusip = d.pop("cusip", UNSET)

        date: Union[CharacteristicDate, Unset] = UNSET
        _date = d.pop("date", UNSET)
        if not isinstance(_date, Unset):
            date = CharacteristicDate.from_dict(cast(Dict[str, Any], _date))

        factor = d.pop("factor", UNSET)

        smm = d.pop("smm", UNSET)

        wac = d.pop("wac", UNSET)

        wala = d.pop("wala", UNSET)

        wam = d.pop("wam", UNSET)

        characteristic_item = cls(
            cusip=cusip,
            date=date,
            factor=factor,
            smm=smm,
            wac=wac,
            wala=wala,
            wam=wam,
        )

        characteristic_item.additional_properties = d
        return characteristic_item

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
