from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.portfolio_prediction_as_of import PortfolioPredictionAsOf
from ..models.prediction_service_get_portfolio_json_body_convention import (
    PredictionService_GetPortfolioJsonBodyConvention,
)
from ..models.prediction_service_get_portfolio_json_body_scenario import (
    PredictionService_GetPortfolioJsonBodyScenario,
)
from ..types import UNSET, Unset

T = TypeVar("T", bound="PortfolioPredictionRequest")


@attr.s(auto_attribs=True)
class PortfolioPredictionRequest:
    """  """

    as_of: Union[PortfolioPredictionAsOf, Unset] = UNSET
    convention: Union[
        Unset, PredictionService_GetPortfolioJsonBodyConvention
    ] = PredictionService_GetPortfolioJsonBodyConvention.PERIOD
    cusips: Union[Unset, List[str]] = UNSET
    scenario: Union[
        Unset, PredictionService_GetPortfolioJsonBodyScenario
    ] = PredictionService_GetPortfolioJsonBodyScenario.BASELINE
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        as_of: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.as_of, Unset):
            as_of = self.as_of.to_dict()

        convention: Union[
            Unset, PredictionService_GetPortfolioJsonBodyConvention
        ] = UNSET
        if not isinstance(self.convention, Unset):
            convention = self.convention

        cusips: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.cusips, Unset):
            cusips = self.cusips

        scenario: Union[Unset, PredictionService_GetPortfolioJsonBodyScenario] = UNSET
        if not isinstance(self.scenario, Unset):
            scenario = self.scenario

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if as_of is not UNSET:
            field_dict["asOf"] = as_of
        if convention is not UNSET:
            field_dict["convention"] = convention
        if cusips is not UNSET:
            field_dict["cusips"] = cusips
        if scenario is not UNSET:
            field_dict["scenario"] = scenario

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        as_of: Union[PortfolioPredictionAsOf, Unset] = UNSET
        _as_of = d.pop("asOf", UNSET)
        if not isinstance(_as_of, Unset):
            as_of = PortfolioPredictionAsOf.from_dict(cast(Dict[str, Any], _as_of))

        convention = None
        _convention = d.pop("convention", UNSET)
        if _convention is not None and _convention is not UNSET:
            convention = PredictionService_GetPortfolioJsonBodyConvention(_convention)

        cusips = cast(List[str], d.pop("cusips", UNSET))

        scenario = None
        _scenario = d.pop("scenario", UNSET)
        if _scenario is not None and _scenario is not UNSET:
            scenario = PredictionService_GetPortfolioJsonBodyScenario(_scenario)

        portfolio_prediction_request = cls(
            as_of=as_of,
            convention=convention,
            cusips=cusips,
            scenario=scenario,
        )

        portfolio_prediction_request.additional_properties = d
        return portfolio_prediction_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
