from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.servicer import Servicer
from ..types import UNSET, Unset

T = TypeVar("T", bound="Servicers")


@attr.s(auto_attribs=True)
class Servicers:
    """ Pool Servicers allocations. """

    cusip: Union[Unset, str] = UNSET
    servicers: Union[Unset, List[Servicer]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        cusip = self.cusip
        servicers: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.servicers, Unset):
            servicers = []
            for servicers_item_data in self.servicers:
                servicers_item = servicers_item_data.to_dict()

                servicers.append(servicers_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if cusip is not UNSET:
            field_dict["cusip"] = cusip
        if servicers is not UNSET:
            field_dict["servicers"] = servicers

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        cusip = d.pop("cusip", UNSET)

        servicers = []
        _servicers = d.pop("servicers", UNSET)
        for servicers_item_data in _servicers or []:
            servicers_item = Servicer.from_dict(servicers_item_data)

            servicers.append(servicers_item)

        servicers = cls(
            cusip=cusip,
            servicers=servicers,
        )

        servicers.additional_properties = d
        return servicers

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
