"""Decorator to enforce named argument only."""
from functools import wraps
from inspect import signature

with_warning = True


def disable_named_argument_warning():
    global with_warning
    with_warning = False


def named_arguments_only(func):
    @wraps(func)
    def inner(*args, **kwargs):
        if len(args) > 2:
            if with_warning:
                sig = signature(func)
                used = []
                for key, value in sig.parameters.items():
                    if key == "self":
                        continue
                    used.append(f"{key}={args[len(used)+1]}")
                    if len(used) == len(args) - 1:
                        break
                print(
                    f"""You are using more than one positional argument. Here are your arguments:
{', '.join(used)}

To get rid of this warning:
from infima_client.util import disable_named_argument_warning
disable_named_argument_warning()

We are recommending using named arguments."""
                )
        return func(*args, **kwargs)

    return inner
